@@ -1,9 +1,9 @@
 #
 # WAJIG - Debian Package Management Front End
 #
-# Manage the files keeping track of changes
+# Keep track of changes between UPDATEs
 #
-# Copyright (c) Graham.Williams@csiro.au
+# Copyright (c) Graham.Williams@togaware.com
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -20,15 +20,17 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
+#
+# TODO
+#
+# Perhaps better to call this "package" rather than "changes"
+
 #------------------------------------------------------------------------
 #
 # Standard python modules
 #
 #------------------------------------------------------------------------
-import os
-import string
-import tempfile
-import socket
+import os,string,tempfile,socket
 
 #------------------------------------------------------------------------
 #
@@ -48,15 +50,15 @@ import perform
 #
 # LOCATIONS
 #
-# Wajig can be run on several machines sharing the same home directories
-# (often through NFS) so we need to have host specific status files. 
-# Also make sure the directories exist.
+# init_dir
+#
+# 	Wajig can be run on several machines sharing the same home
+#	directories (often through NFS) so we need to have host specific
+#	status files. Make sure the directories exist.
+#
 #------------------------------------------------------------------------
-init_dir = os.path.expanduser("~/.wajig/")
-if not os.path.exists(init_dir): os.mkdir(init_dir)
-init_dir = init_dir + socket.gethostname()
-if not os.path.exists(init_dir): os.mkdir(init_dir)
-
+init_dir = os.path.expanduser("~/.wajig/") + socket.gethostname()
+if not os.path.exists(init_dir): os.makedirs(init_dir)
 #
 # Temporarily, remove old files from .wajig
 # After a few versions remove this code.
@@ -68,10 +70,53 @@ if os.path.exists(tmp_dir + "/Available.prv"):
     os.rename(tmp_dir + "/Available.prv", init_dir + "/Available.prv")
 if os.path.exists(tmp_dir + "/Installed"): 
     os.rename(tmp_dir + "/Installed", init_dir + "/Installed")
-
+#
+# TODO 23 Aug 2003
+#
+# Perhaps the only file that wajig needs to cache itself is
+# Available.prv - the other two can be generated dynamically.
+# Installed is now like this. Available can be done.
+# This would mean essentially half the space usage (Installed
+# is quite small at 30K, while Available and Available.prv
+# were 280K). This is significant for a user who manages
+# many hosts.
+#
+# If this is the case, then don't worry about creating a host
+# subdirectory in ~/.wajig.  Simply use host in filename,
+# unless init_dir is used elsewhere?????? Perhaps keep as folder
+# since tempfiles are created there.
+#
+# TODO Work to use bzip2 files for available and previous.
+# Then bunzip2 to temporary files when needed!
+# Disk usage goes from 274K to 83K.
+#
 available_file = init_dir + "/Available"
 previous_file  = init_dir + "/Available.prv"
-installed_file = init_dir + "/Installed"
+#
+# No longer using the Installed file so remove it. 23 Aug 2003
+#
+if os.path.exists(init_dir + "/Installed"): 
+    os.remove(init_dir + "/Installed")
+#
+# Packages cache
+#
+# This was added when some /var/lib/dpkg/available stopped being
+# updated for some reason - perhaps new version of dpkg or apt
+# (Jun/Jul 03). File can be big, but hope that's okay.
+#
+# BUT no it is not okay! For large installations, this creates
+# additional 11MB per host per user of wajig :-(
+# Generateion of the file is quick (0.5s) so just build it
+# as needed in /tmp. 23 Aug 2003
+#
+#packages_file  = init_dir + "/Packages"
+#
+# Remove the large cache file - this should only be needed
+# for a little while as 0.3.26 replaced 0.3.25 pretty quickly.
+# 23 Aug 2003
+#
+if os.path.exists(init_dir + "/Packages"):
+    os.remove(init_dir + "/Packages")
 
 #
 # Set the temporary directory to the init_dir.  
@@ -79,16 +124,6 @@ installed_file = init_dir + "/Installed"
 #
 tempfile.tempdir = init_dir
 
-def get_installed_filename():
-    """Obtain the name of the file containing list of installed packages.
-
-    Arguments:
-
-    Returns:
-    string	The filename"""
-
-    return installed_file
-
 def get_available_filename():
     """Obtain the name of the file containing list of available packages.
 
@@ -114,7 +149,7 @@ def get_previous_filename():
 # UPDATE AVAILABLE
 #
 #------------------------------------------------------------------------
-def update_available(noreport=0):
+def update_available(noreport=False):
     """Generate current list of available packages, backing up the old list.
 
     Arguments:
@@ -129,76 +164,64 @@ def update_available(noreport=0):
     temporary_file = tempfile.mktemp()
 
     os.rename(available_file, temporary_file)
-
     command = "apt-cache dumpavail " +\
 	      "| egrep '^(Package|Version):' " +\
 	      "| tr '\n' ' '" +\
 	      "| perl -p -e 's|Package: |\n|g; s|Version: ||g'" +\
- 	      "| sort | tail +2 > " + available_file
-    perform.execute(command, noquiet=1, root=1)
-
+ 	      "| sort | tail +2 | sed 's| $||' > " + available_file
+    perform.execute(command, noquiet=True) # root is not required.
     os.rename(temporary_file, previous_file)
 
     available_packages = len(open(available_file).readlines())
     previous_packages = len(open(previous_file).readlines())
     diff = available_packages - previous_packages
-    if noreport == 0:
+    newest = perform.execute("join -v 1 -t' '  " +
+                             available_file + " " +
+                             previous_file +
+                             " | wc -l", pipe=True).readlines()[0].strip()
+    if not noreport:
         if diff < 0:
             direction = str(0-diff) + " down on"
         elif diff == 0:
             direction = "the same as"
         else:
             direction = str(diff) + " up on"
-        #print "There are " + str(available_packages) + " packages now " +\
+        #print "There are " + str(available_packages) + " packages now " +
 	#      "available which is " + direction + " the previous count."
-        print "This is " + direction + " the previous count."
+        print "This is " + direction + " the previous count",
+        print "with " + newest + " new packages."
 
 #------------------------------------------------------------------------
 #
-# UPDATE INSTALLED
+# GET INSTALLED COMMAND STR
 #
 #------------------------------------------------------------------------
-def update_installed():
-    """Generate current list of installed packages and their status.
-
-    Arguments:
-
-    Returns:"""
-
-    if not os.path.exists(installed_file):
-	f = open(installed_file,"w")
-	f.close()
-
+def gen_installed_command_str():
+    """Generate command to list installed packages and their status."""
     command = "cat /var/lib/dpkg/status | " +\
 	      "egrep '^(Package|Status|Version):' | " +\
               "awk '/^Package: / {pkg=$2} " +\
 	      "     /^Status: / {s1=$2;s2=$3;s3=$4}" +\
 	      "     /^Version: / {print pkg,$2,s1,s2,s3}' | " +\
-	      "grep 'ok installed' | " +\
-	      "awk '{print $1,$2}' | sort > " + installed_file
-
-    perform.execute(command, noquiet=1)
+	      "grep 'ok installed' | awk '{print $1,$2}' | sort "
+    return(command)
 
 def count_upgrades():
-    f = tempfile.mktemp()
+    """Return as a string the number of new upgrades since last update."""
+    ifile = tempfile.mktemp()
+    perform.execute(gen_installed_command_str() + " > " + ifile, noquiet=1)
     command = "join " + previous_file + " " + available_file + " |" +\
-                  "awk '$2 != $3 {print}' |" +\
-                  "join - " + installed_file + " |" +\
-                  "awk '$4 != $3 {print}' | wc -l | awk '{print $1}' > " + f
-    perform.execute(command, noquiet=1)
-    fo = open(f)
-    updated = string.split(fo.read())[0];
-    print "There are " + str(updated) + " new upgrades"
-    if os.path.exists(f): os.remove(f)
+              "awk '$2 != $3 {print}' | join - " + ifile + " |" +\
+              "awk '$4 != $3 {print}' | wc -l | awk '{print $1}' "
+    count = string.split(perform.execute(command, noquiet=1, pipe=1).read())[0]
+    if os.path.exists(ifile): os.remove(ifile)
+    return count
 
 def reset_files():    
     if os.path.exists(available_file): os.remove(available_file)
     if os.path.exists(previous_file): os.remove(previous_file)
-    if os.path.exists(installed_file): os.remove(installed_file)
     update_available(noreport=1)
     update_available(noreport=1)
-    update_installed()
-
 
 #------------------------------------------------------------------------
 #
@@ -213,13 +236,10 @@ def ensure_initialised():
     Returns:"""
 
     if not os.path.exists(available_file): reset_files()
-    if not os.path.exists(installed_file): update_installed()
-
-    
 
 #------------------------------------------------------------------------
 # 
-# Dictionaries of available, installed, and previously available
+# Dictionaries of available, installed, and previously available packages.
 #
 #------------------------------------------------------------------------
 available_list = {}
@@ -227,11 +247,7 @@ previous_list = {}
 installed_list = {}
 
 def load_dictionaries():
-    """Read the wajig packages files to create dictionaries of available/installed pkgs
-
-    Arguments:
-
-    Returns:"""
+    """Create dictionaries of avail/installed packages for in-memory tasks."""
 
     ensure_initialised()
     
@@ -243,7 +259,8 @@ def load_dictionaries():
     for i in range(0,len(pfile)):
         previous_list[string.split(pfile[i])[0]] = string.split(pfile[i])[1]
 
-    ifile = open(installed_file,"r").readlines()
+    ifile = perform.execute(gen_installed_command_str(),
+                            noquiet=1, pipe=1).readlines()
     for i in range(0,len(ifile)):
         installed_list[string.split(ifile[i])[0]] = string.split(ifile[i])[1]
 
@@ -267,16 +284,6 @@ def get_previous_list():
 
     return previous_list
 
-def get_installed_list():
-    """Obtain the dictionary of installed packages.
-
-    Arguments:
-
-    Returns:
-    dictionary	Installed packages"""
-
-    return installed_list
-
 def get_available_version(pkg):
     """Obtain the package's available version number.
 
@@ -298,13 +305,8 @@ def get_previous_version(pkg):
     return previous_list[pkg]
 
 def get_installed_version(pkg):
-    """Obtain the package's installed version number.
-
-    Arguments:
-
-    Returns:
-    string	Installed version"""
-
+    """Return, as string, the package's installed version number."""
+    # TODO: Make sure the dictionary has been loaded.
     return installed_list[pkg]
 
 def get_new_available():
@@ -314,7 +316,7 @@ def get_new_available():
 
     Returns:
     list	New packages"""
-
+    load_dictionaries()
     new_list = []
     for pkg in available_list.keys():
         if not previous_list.has_key(pkg):
@@ -328,7 +330,7 @@ def get_new_upgrades():
 
     Returns:
     list	Newly upgraded packages"""
-
+    load_dictionaries()
     upgraded_list = []
     apt_pkg.InitSystem(); # Not sure why!
     for pkg in installed_list.keys():
@@ -347,6 +349,7 @@ def get_to_upgrade():
     Returns:
     list	Upgradeable packages"""
 
+    load_dictionaries()
     upgraded_list = []
     apt_pkg.InitSystem(); # Not sure why!
     for pkg in installed_list.keys():
@@ -355,3 +358,48 @@ def get_to_upgrade():
                                       installed_list[pkg]) > 0:
             upgraded_list.append(pkg)
     return upgraded_list
+
+def get_status(pkg):
+    p = perform.execute("dpkg --status " + pkg, pipe=True)
+    pkginfo = apt_pkg.ParseTagFile(p)
+    pkginfo.Step()
+    return pkginfo.Section.get("Status")
+
+def get_dependees(pkg):
+    """Return a list of other installed pkgs that depend on PKG."""
+    pkginfo = perform.execute("apt-cache rdepends " + pkg, pipe=True)
+    dp = []
+    # Watch for changes to apt-cache output.
+    if pkginfo.next().strip() <> pkg:
+        print "wajig: unexpected result from apt-cache - submit a bug report"
+    if pkginfo.next().strip() <> "Reverse Depends:":
+        print "wajig: unexpected result from apt-cache - submit a bug report"
+    for l in pkginfo:
+        pn = l.strip().split(',')[0].replace('|','')
+        dp.append(pn)
+    #
+    # If dp is too long, don't check install status.
+    # Just assume installed.
+    # libc6 (over 7000), for exapmle, takes far too long.
+    # Really just need to find first pkg that is installed
+    # that depends on this pkg to rule it out.
+    #
+    np = []
+    if len(dp) < 20:
+        for p in dp:
+            if get_status(p).find("not-installed") < 0:
+                np.append(p)
+    return np
+
+def get_dependencies(pkg):
+    """Return a list of installed packages that PKG depends on."""
+    pkginfo = perform.execute("apt-cache depends " + pkg, pipe=True)
+    dp = []
+    # Watch for changes to apt-cache output.
+    if pkginfo.next().strip() <> pkg:
+        print "wajig: unexpected result from apt-cache - submit a bug report"
+    # Find package names. Ignore "<name>" as these are not installed.
+    for l in pkginfo:
+        if l.find(":") >= 0 and l.find("<") < 0:
+            dp.append(l.split()[1])
+    return dp
