@@ -3,7 +3,7 @@
 #
 # Implementation of all commands
 #
-# Copyright (c) Graham.Williams@csiro.au
+# Copyright (c) Graham.Williams@togaware.com
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -20,17 +20,10 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-#------------------------------------------------------------------------
 #
 # Standard python modules
 #
-#------------------------------------------------------------------------
-import os
-import re
-import string
-import sys
-import tempfile
-import signal
+import os,re,string,sys,tempfile,signal
 #
 # When writing to a pipe where there is no reader (e.g., when
 # output is directed to head or to less and the user exists from less
@@ -39,59 +32,53 @@ import signal
 #
 signal.signal(signal.SIGPIPE,signal.SIG_DFL)
 
-#------------------------------------------------------------------------
 #
 # APT module
 #
-#------------------------------------------------------------------------
 import apt_pkg
 
-#------------------------------------------------------------------------
 #
 # Wajig modules
 #
-#------------------------------------------------------------------------
 import changes
 import perform
 
-#------------------------------------------------------------------------
 #
 # Global Variables
 #
-#------------------------------------------------------------------------
-installed_file = changes.get_installed_filename()
 available_file = changes.get_available_filename()
 previous_file  = changes.get_previous_filename()
 
-#------------------------------------------------------------------------
 #
 # Interface Variables
 #
-#------------------------------------------------------------------------
-
 verbose = 0
 
 def set_verbosity_level(new_level):
     global verbose
     verbose = new_level
 
-#------------------------------------------------------------------------
 #
-# COMMAND
+# SUPPORT
 #
-#------------------------------------------------------------------------
-def do_command(command, root=0, update=0):
-    """Simply perform the command and optionally update Installed list
-
-    Arguments:
-    command	Command to perform
-    root	Whether the command needs root access
-    update	Whether to also update the list of installed packages
-
-    Returns:"""
-
-    perform.execute(command,root=root)
-    if update > 0: changes.update_installed()
+def ping_host(hostname):
+    #
+    # Check if fping is installed.
+    #
+    if perform.execute("fping localhost 2>/dev/null >/dev/null",
+                       display=0) != 0:
+        print "JIG Warning: fping was not found. " +\
+              "Consider installing the package fping.\n"
+    #
+    # Check if we can talk to the HOST
+    #
+    elif perform.execute("fping " + hostname + " 2>/dev/null >/dev/null",
+                         display=0) != 0:
+    	print "JIG Warning: Could not contact the Debian server at\n" +\
+              "             " + hostname + """
+             Perhaps it is down or you are not connected to the network.
+             JIG will continue to try to get the information required.
+"""
 
 #------------------------------------------------------------------------
 #
@@ -116,13 +103,16 @@ def do_describe(packages):
     Returns:"""
 
     #
-    # From where do we get the information?
+    # From where do we get information about the packages?
+    #
     #   /var/lib/dpkg/available		dpkg's idea of available
     #	/var/cache/apt/available	apt's  idea of available
     #
-    # TODO does apt_pkg provide a way to get available apt-cache directly?
+    # TODO does apt_pkg provide a way to get available apt-cache directly,
+    # and does not require root access - otherwise I need to do it through
+    # sudo with external commands.
     #
-    # I though to move to /var/cache/apt/available so I don't need to
+    # I did think to move to /var/cache/apt/available so I don't need to
     # use dpkg's copy which is not updated by apt-get update.
     # Hoever, "dselect update" does update it after doing
     # "apt-get update". 
@@ -139,11 +129,20 @@ def do_describe(packages):
     #  apt-cache dumpavail > /tmp/avail
     #  dpkg --update-avail /tmp/avail
     #
-    #
     # avail  = apt_pkg.ParseTagFile(open("/var/cache/apt/available","r"));
     # avail  = apt_pkg.ParseTagFile(open("/var/cache/apt/pkgcache.bin","r"));
-    avail  = apt_pkg.ParseTagFile(open("/var/lib/dpkg/available","r"));
-    status = apt_pkg.ParseTagFile(open("/var/lib/dpkg/status","r"));
+    # avail  = apt_pkg.ParseTagFile(open("/var/lib/dpkg/available","r"))
+    #
+    # Simplest solution seems to be to pipe details of all packages
+    # from apt-cache (which needs sudo). 23 Aug 2003.
+    #
+    # However, as of 11 Nov 2003 at least, apt-cache seems user
+    # accessible on festiva, alpine, cultus, fairmont. So remove
+    # root=1.
+    #
+    command = "apt-cache dumpavail"
+    packages_pipe = perform.execute(command, noquiet=True, pipe=True)
+    avail = apt_pkg.ParseTagFile(packages_pipe)
     #
     # Record the descriptions
     #
@@ -158,15 +157,6 @@ def do_describe(packages):
             if not describe_list.has_key(package_name):
                 describe_list[package_name] = package_description
     #
-    # Check for information in the Status list
-    #
-    while status.Step():
-        if (status.Section.get("Package") in packages):
-            package_name = status.Section.get("Package")
-            package_description = status.Section.get("Description")
-            if not describe_list.has_key(package_name):
-                describe_list[package_name] = package_description
-    #
     # Print out the one line descriptions. Should it be sorted?
     # If not sorted it should be same order as on the command line.
     #
@@ -177,10 +167,27 @@ def do_describe(packages):
     # through the value of "verbose"
     #
     if len(pkgs) == 0:
-        print "No packages to describe"
+        print "No packages found from those known to be available or installed"
     elif verbose == 0:
         print "%-24s %s" % ("Package", "Description")
         print "="*24 + "-" + "="*51
+
+#
+# We could start using the grep-dctrl package to get the information.
+# Almost works (except when package is not found in one of status
+# or available - still prints package name)
+#
+#        command = "(:"
+#        for pkg in packages:
+#            command += "; printf \"%-24s \" " + pkg +\
+#                       "; grep-status -XP -s Description " + pkg +\
+#                       "| head -1 | cut -d' ' -f2- "
+#            command += "; printf \"%-24s \" " + pkg +\
+#                       "; grep-available -XP -s Description " + pkg +\
+#                       "| head -1 | cut -d' ' -f2- "
+#        command += ") | sort"
+#        perform.execute(command)
+        
         for pkg in pkgs:
             # Only print that first line, but check that there
             # is a description available.
@@ -199,7 +206,7 @@ def do_describe(packages):
         #
         package_names = perform.concat(packages)
         command = "apt-cache show " + package_names
-        perform.execute(command,root=1)
+        perform.execute(command)
 
 #------------------------------------------------------------------------
 #
@@ -214,9 +221,8 @@ def do_describe_new():
     Returns:"""
 
     #
-    # Load the dictionaries from file then describe each new one.
+    # Describe each new package.
     #
-    changes.load_dictionaries();
     new_pkgs = changes.get_new_available();
     if len(new_pkgs) == 0:
         print "No new packages"
@@ -238,7 +244,6 @@ def do_download(packages):
 
     command = "apt-get --download-only install " + perform.concat(packages)
     perform.execute(command, root=1)
-    changes.update_installed()
 
 #------------------------------------------------------------------------
 #
@@ -251,14 +256,15 @@ def do_force(packages):
     """Force the installation of a package.
 
     This is useful when there is a conflict of the same file from
-    multiple packages.
+    multiple packages or when a dependency is not installed for
+    whatever reason.
 
     Arguments:
     packages	The packages to be installed (must be in the cache)
 
     Returns:"""
 
-    command = "dpkg --install --force overwrite "
+    command = "dpkg --install --force overwrite --force depends "
     if re.match(".*\.deb$", packages[0]):
         for pkg in packages:
             if os.path.exists(pkg):
@@ -279,8 +285,6 @@ def do_force(packages):
                       " | egrep '^" + pkg + "_' | sort | tail -1)"
         perform.execute(command,root=1)
 
-    changes.update_installed()
-
 #------------------------------------------------------------------------
 #
 # HOLD
@@ -300,6 +304,10 @@ def do_hold(packages):
         command = "echo \"" + p + " hold\" | " +\
 	          perform.setroot + " dpkg --set-selections"
 	perform.execute(command)
+    print "The following packages are on hold:"
+    perform.execute("dpkg --get-selections | egrep 'hold$' | cut -f1")
+
+    
 
 #------------------------------------------------------------------------
 #
@@ -327,6 +335,21 @@ def do_install(packages):
     #
     # Currently only a single URL is allowed. Should this be generalised?
     #
+
+    #
+    # If reading from standard input generate the list of packages.
+    #
+    if len(packages) == 1 and packages[0] == "-":
+        packages = string.split(
+            string.join(map(string.strip,sys.stdin.readlines())))
+    #
+    # If reading packages from file generate list of packages.
+    #
+    elif len(packages) == 2 and packages[0] == "-f":
+        packages = string.split(string.join(map(string.strip,(open(packages[1])).readlines())))
+    #
+    # Check if a specific web location was specified.
+    #
     if re.match("(http|ftp)://", packages[0]):
         if len(packages) > 1:
             print "wajig: Error: install URL allows only one URL, not " +\
@@ -342,6 +365,9 @@ def do_install(packages):
             print "Wajig: The location " + packages[0] +\
                   " was not found. Check and try again."
             return()
+    #
+    # Check if a specific .DEB file was specified
+    #
     elif re.match(".*\.deb$", packages[0]):
         command = "dpkg -i "
         for pkg in packages:
@@ -357,46 +383,143 @@ def do_install(packages):
                       "       Please confirm location and try again."
                 return()
         perform.execute(command, root=1)
+    #
+    # Check if a "/+" is in a package name then use the following distribution
+    # for all packages! We might not want this previsely if there are multiple
+    # packages listed and only one has the /+ notation. So do it only for the
+    # specified one. I have introduced this notation myself, extending
+    # the apt-get "/" notation. "+" by itself won't work since "+" can
+    # appear in a package name, and it is okay if a distribution name starts
+    # with "+" since you just include two "+"'s then.
+    #
+    # TODO
+    #
+    # Currently only do this for the first package........
+    #
+#     elif re.match(".*/+.*", packages[0]):
+#         print "HI"
+#         (packages[0], release) = re.compile(r'/\+').split(packages[0])
+# 	command = "apt-get --target-release %s install %s" %\
+#                   (release, perform.concat(packages))
+# 	perform.execute(command, root=1)
     else:
 	command = "apt-get install " + perform.concat(packages)
 	perform.execute(command, root=1)
-    changes.update_installed()
 
 #------------------------------------------------------------------------
 #
-# LISTNAMES
+# INSTALL SUGGEST
 #
 #------------------------------------------------------------------------
-def do_listinstalled(pattern):
-    """Print list of installed packages.
+def do_install_suggest(packages, type):
+    """Install packages suggested by the listed packages.
 
     Arguments:
+    packages	List of packages 
 
     Returns:"""
 
-    command = "dpkg --get-selections | awk '$2 ~/^install$/ {print $1}'"
+    #
+    # If reading from stanard input generate the list of packages
+    #
+    if len(packages) == 1 and packages[0] == "-":
+        packages = string.split(
+            string.join(map(string.strip,sys.stdin.readlines())))
+        
+    #
+    # For each package obtain the list of suggested packages
+    #
+    suggest_list = ""
+    #
+    # As root obtain the details of each available package,
+    # using apt-cache (requires sudo). 23 Aug 2003.
+    #
+    # Remove root=1 from execute since apt-cache seems accessible to
+    # user.
+    #
+    command = "apt-cache dumpavail"
+    packages_pipe = perform.execute(command, noquiet=True, pipe=True)
+    avail = apt_pkg.ParseTagFile(packages_pipe)
+    #
+    # Check for information in the Available list
+    #
+    while avail.Step():
+        if (avail.Section.get("Package") in packages):
+            sug = avail.Section.get("Suggests")
+            rec = avail.Section.get("Recommends")
+            if type == "Suggests" or type == "Both":
+                if sug: suggest_list = suggest_list+" "+sug
+            if type == "Recommends" or type == "Both":
+                if rec: suggest_list = suggest_list+" "+rec
+    #
+    # Remove version information
+    #
+    suggest_list = re.sub('\([^)]*\)', '', suggest_list)
+    #
+    # Remove the commas from the list
+    #
+    suggest_list = re.sub(',', '', suggest_list)
+    #
+    # Deal with alternatives.
+    # For now simply ignore all alternatives.
+    # Should prompt user to select one?
+    #
+    suggest_list = re.sub(' *[a-z0-9-]* *\| *[a-z0-9-]* *', ' ', suggest_list)
+    #
+    # Remove duplicates
+    #
+    suggest_list = suggest_list + " "
+    for i in string.split(suggest_list):
+        suggest_list = i + " " + re.sub(" "+i+" ", ' ', suggest_list)
+    #
+    # For the list of supplied packages and the list of suggested packages
+    # do an install
+    #
+    # I used to ask but get apt-get to do the asking.
+    #
+    #if len(string.strip(suggest_list)) > 0:
+    #  print "Installing: " + suggest_list
+    #  cont = raw_input('Continue (Y/n)? ')
+    #else:
+    #    cont = ""
+    #if cont == "" or cont == "Y" or cont == "y" or cont == "yes":
+    command = "apt-get  --show-upgraded install " \
+              + perform.concat(packages) + suggest_list
+    perform.execute(command, root=1)
 
+#------------------------------------------------------------------------
+#
+# LIST INSTALLED
+#
+#------------------------------------------------------------------------
+def do_listinstalled(pattern):
+    """Print list of installed packages."""
+    command = "dpkg --get-selections | awk '$2 ~/^install$/ {print $1}'"
     if len(pattern) == 1:
-	command = command + " | grep " + pattern[0] + " | sort"
+	command = command + " | grep -- " + pattern[0] + " | sort"
     perform.execute(command)
 
 #------------------------------------------------------------------------
 #
-# LISTNAMES
+# LIST NAMES
 #
 #------------------------------------------------------------------------
-def do_listnames(pattern):
+def do_listnames(pattern, pipe=False):
     """Print list of known package names.
 
     Arguments:
 
     Returns:"""
-
+    #
+    # If user can't access /etc/apt/sources.list then must do this with
+    # sudo or else most packages will not be found.
+    #
+    needsudo = not os.access("/etc/apt/sources.list", os.R_OK)
     if len(pattern) == 0:
         command = "apt-cache pkgnames | sort"
     else:
-	command = "apt-cache pkgnames | grep " + pattern[0] + " | sort"
-    perform.execute(command)
+	command = "apt-cache pkgnames | grep -- " + pattern[0] + " | sort"
+    return perform.execute(command, root=needsudo, pipe=pipe)
 
 #------------------------------------------------------------------------
 #
@@ -413,14 +536,87 @@ def do_new():
     print "%-24s %s" % ("Package", "Available")
     print "="*24 + "-" + "="*16
     #
-    # Load the dictionaries from file then list each one and it's version
+    # List each package and it's version
     #
-    changes.load_dictionaries();
     new_pkgs = changes.get_new_available();
     new_pkgs.sort()
     for i in range(0,len(new_pkgs)):
         print "%-24s %s" % (new_pkgs[i], changes.get_available_version(new_pkgs[i]))
                    
+#------------------------------------------------------------------------
+#
+# NEWS
+#
+#------------------------------------------------------------------------
+def map_sources(packages):
+    """Return Source package name for each package in PACKAGES"""
+    Parse = apt_pkg.ParseTagFile(open("/var/lib/dpkg/available","r"))
+    sources = []
+    while Parse.Step():
+        # May want to look to terminate as soon as all packages
+        # have been found - to be more efficient
+        if Parse.Section.get("Package") in packages:
+            if Parse.Section.get("Source"):
+                sources += [Parse.Section.get("Source")]
+            else:
+                sources += [Parse.Section.get("Package")]
+    return sources
+
+def do_news(packages):
+    """List latest news for each package.
+
+    Arguments:
+    packages	List the latest news for each.
+
+    Returns: News items
+    """
+    ping_host("packages.qa.debian.org")
+    results = tempfile.mktemp()
+    packages = map_sources(packages)
+    for p in packages:
+        # Pipe through Lynx to map HTML entities like &quot;
+        command = "wget --timeout=60 --output-document=-" +\
+                  " http://packages.qa.debian.org/" + p[0] + "/" + p + \
+                  "/news/1.html 2> /dev/null" +\
+                  "| lynx --dump --stdin " +\
+                  " > " + results
+        perform.execute(command)
+        print "="*((70-len(p))/2) + " " + p + " " + "="*((70-len(p))/2)
+        command = "cat " + results + " | " +\
+                  "awk 'BEGIN{pr=0}/^Description:/{pr=1}/^Files:/{pr=0}" +\
+                  "pr==1{print;next}{next}' "
+        perform.execute(command)
+        if os.path.exists(results): os.remove(results)
+
+
+#------------------------------------------------------------------------
+#
+# CHANGELOG
+#
+#------------------------------------------------------------------------
+
+def do_changelog(packages):
+    """List latest changelog for each package.
+
+    Arguments:
+    packages	List the latest changelog for each.
+
+    Returns: Changelogs
+    """
+    ping_host("people.debian.org")
+    results = tempfile.mktemp()
+    packages = map_sources(packages)
+    for p in packages:
+        # Pipe through Lynx to map HTML entities like &quot;
+        print "="*((70-len(p))/2) + " " + p + " " + "="*((70-len(p))/2)
+        command = "wget --timeout=60 --output-document=-" +\
+                  " http://people.debian.org/~noel/changelogs/pool/main/" +\
+                  p[0] + "/" + p + "/" + p + "_" +\
+                  "`egrep '^" + p + " ' " + available_file +\
+                  " | cut -d' ' -f2`" +\
+                  "/changelog 2> /dev/null"
+        perform.execute(command)
+
 #------------------------------------------------------------------------
 #
 # NEWUPGRADES
@@ -436,7 +632,6 @@ def do_newupgrades():
     #
     # Load the dictionaries from file then list each one and it's version
     #
-    changes.load_dictionaries();
     new_upgrades = changes.get_new_upgrades();
     if len(new_upgrades) == 0:
         print "No new upgrades"
@@ -449,6 +644,77 @@ def do_newupgrades():
                             changes.get_available_version(new_upgrades[i]), \
                             changes.get_installed_version(new_upgrades[i]))
 
+def do_removedepend(pkg, purge=False):
+    print "Checking dependencies. Please wait..."
+    toremove = pkg
+    #
+    # Obtain list of dependencies.
+    #
+    dependencies = changes.get_dependencies(pkg)
+    #
+    # For each dependency, find dependees that are installed.
+    # If only one is this pkg, then add it to the removal list.
+    #
+    if dependencies:
+        for p in dependencies:
+            # Check if the supplied pkg is the only dependee
+            dp = changes.get_dependees(p)
+            if changes.get_dependees(p) == [pkg]:
+                toremove += " " + p
+    #
+    # Now do the actual removal
+    #
+    print "Removing:", toremove
+    if purge:
+        perform.execute("dpkg --purge " + toremove, root=1)
+    else:
+        perform.execute("apt-get remove " + toremove, root=1)
+
+
+########################################################################
+# SIZE
+#
+def do_size(packages, size):
+    """Print sizes for pkg in list PACAKAGES with size greater than SIZE."""
+    #
+    # Work with the list of installed packages
+    # (I think status has more than installed?)
+    #
+    status = apt_pkg.ParseTagFile(open("/var/lib/dpkg/status","r"));
+    #
+    # Initialise the sizes dictionary.
+    #
+    size_list = {}
+    status_list = {}
+    #
+    # Check for information in the Status list
+    #
+    while status.Step():
+        if not packages or status.Section.get("Package") in packages:
+            package_name   = status.Section.get("Package")
+            package_size   = status.Section.get("Installed-Size")
+            package_status = re.split(" ", status.Section.get("Status"))[2]
+            if package_size and int(package_size) > size:
+                if not size_list.has_key(package_name):
+                    size_list[package_name] = package_size
+                    status_list[package_name] = package_status
+    #
+    # Print out the one line descriptions.
+    #
+    pkgs = size_list.keys()
+    pkgs.sort()
+    #
+    # Output
+    #
+    if len(pkgs) == 0:
+        print "No packages found from those known to be available or installed"
+    else:
+        print "%-30s %s       %s" % ("Package", "Size", "Status")
+        print "="*30 + "-" + "="*10 + "-" + "="*35
+        for pkg in pkgs:
+            print "%-30s %6s     %-20s" \
+                  % (pkg, size_list[pkg], status_list[pkg])
+
 #------------------------------------------------------------------------
 #
 # STATUS
@@ -475,33 +741,39 @@ def do_status(packages):
     #
     #   command = "dpkg --list | " +\
     #             "awk '{print $2,$1}' | " +\
-
-    command = "dpkg --get-selections | " +\
-	      "join - " + installed_file + " | " +\
+    #
+    # Generate a temporary file of installed packages.
+    #
+    ifile = tempfile.mktemp()
+    perform.execute(changes.gen_installed_command_str() + " > " + ifile,
+                    noquiet=1)
+    #
+    # Build the command to list the status of installed packages.
+    #
+    command = "dpkg --get-selections | join - " + ifile + " | " +\
               "join -a 1 - " + previous_file + " | " +\
 	      "awk 'NF==3 {print $0, \"N/A\"; next}{print}' | " +\
 	      "join -a 1 - " + available_file + " | " +\
 	      "awk 'NF==4 {print $0, \"N/A\"; next}{print}' | "
     if len(packages) > 0:
-        command = command + "egrep '^($"
+        # Use grep, not egrep, otherwise g++ gets lost, for example!
+        command = command + "grep '^\($"
         for i in packages: 
-            command = command + " |" + i
-        command = command + " )' |"
+            command = command + " \|" + i
+        command = command + " \)' |"
 
     command = command +\
 	      "awk '{printf(\"%-20s\\t%-15s\\t%-15s\\t%-15s\\t%-2s\\n\", " +\
 	      "$1, $3, $4, $5, $2)}'"
     perform.execute(command)
-
     #
     # Check whether the package is not in the installed list, and if not
     # list its status appropriately.
     #
     for i in packages: 
-        if os.system("egrep '^" + i + " ' " + installed_file +\
-	               " >/dev/null"):
+        if os.system("egrep '^" + i + " ' " + ifile + " >/dev/null"):
             #
-	    # Package is not installed
+	    # Package is not installed.
 	    #
 	    command = \
               "join -a 2 " + previous_file + " " + available_file + " | " +\
@@ -511,8 +783,10 @@ def do_status(packages):
 	      " | awk '{printf(\"%-20s\\t%-15s\\t%-15s\\t%-15s\\n\", " +\
 	      "$1, \"N/A\", $2, $3)}'"
 	    perform.execute(command)
-
-
+    #
+    # Tidy up - remove the "installed file"
+    #
+    if os.path.exists(ifile): os.remove(ifile)
 
 #------------------------------------------------------------------------
 #
@@ -537,9 +811,8 @@ def do_toupgrade():
     print "%-24s %-24s %s" % ("Package", "Available", "Installed")
     print "="*24 + "-" + "="*24 + "-" + "="*24 
     #
-    # Load the dictionaries from file then list each one and it's version
+    # List each upgraded pacakge and it's version.
     #
-    changes.load_dictionaries();
     to_upgrade = changes.get_to_upgrade();
     to_upgrade.sort()
     for i in range(0,len(to_upgrade)):
@@ -547,12 +820,35 @@ def do_toupgrade():
                             changes.get_available_version(to_upgrade[i]), \
                             changes.get_installed_version(to_upgrade[i]))
     
+#------------------------------------------------------------------------
+#
+# edd 03 Sep 2003  unhold patch based on hold semantics
+# UNHOLD
+#
+#------------------------------------------------------------------------
+def do_unhold(packages):
+    """Remove packages from hold (they will again be upgraded).
+    
+    Arguments:
+    packages   List of packages.
+    
+    Returns:"""
+
+    for p in packages:
+        # The dpkg needs sudo but not the echo.
+        # Maybe sensible to have all sudo's in this file?
+        command = "echo \"" + p + " install\" | " +\
+                  perform.setroot + " dpkg --set-selections"
+        perform.execute(command)
+    print "The following packages are still on hold:"
+    perform.execute("dpkg --get-selections | egrep 'hold$' | cut -f1")
+        
 #------------------------------------------------------------------------
 #
 # UPDATE
 #
 #------------------------------------------------------------------------
-def do_update(query=0):
+def do_update(query=0, quiet=0):
     """Perform.Execute an update of the available packages
 
     Arguments:
@@ -594,7 +890,11 @@ likely that your Packages files are out of date. Doing an UPDATE is optional
     # 02/03/26 Go back to dselect - The apt available file is not updated
     # by apt-get so new was not finding the package descriptions.
     #
-    if perform.execute("dselect update", root=1) == 0:
+    if quiet:
+        quietopt = "| egrep -v '(http|ftp)'"
+    else:
+        quietopt = ""
+    if perform.execute("dselect update " + quietopt, root=1) == 0:
     # if perform.execute("apt-get update", root=1) == 0:
 	#
 	# Only update the available list if the UPDATE succeeded
@@ -603,11 +903,12 @@ likely that your Packages files are out of date. Doing an UPDATE is optional
 	#
 	# How many new upgrades are there?
 	#
-	changes.count_upgrades()
+        print "There are " + changes.count_upgrades() + " new upgrades"
     else:
         print "Wajig: The update failed for some reason."
         print "       Have you configured dselect lately?"
         print "       It may need updating to use apt."
+        print "       Or perhaps you purposely interrupted me."
 
         exit(1);
 
@@ -658,68 +959,196 @@ def do_whichpkg(filename):
 
     Returns:"""
 
+    ping_host("packages.debian.org")
     #
-    # Thought the following was it once but no, it only searches installed
-    # packages.
+    # Print out a suitable heading
     #
-    # command = "dpkg -S " + filename
-    # perform.execute(command)
+    print "%-59s %-17s" % ("File Path", "Package")
+    print "="*59 + "-" + "="*17
+    sys.stdout.flush()
 
-    command = "fping packages.debian.org 2>&1 >/dev/null"
+    #
+    # Get the local information first.
+    #
+    # Could ensure that we don't repeat information but for now let's
+    # just get the information out!
+    #
+    # Also need to check for diversions (like when mutt and mutt-utf8 are
+    # both installed.
+    #
+    command = "dpkg -S " + filename + " 2>/dev/null " +\
+              "| sed 's|: *|:|' " +\
+              "| grep -v '^diversion by ' " +\
+              "| awk -F: '{printf(\"%-59s %-17s\\n\", $2, $1)}'"
+    perform.execute(command)
+
+    #
+    # Now obtain the information from the Debian server
+    #
+    results = tempfile.mktemp()
+    command = "wget --timeout=60 --output-document=" + results +\
+              " http://packages.debian.org/cgi-bin" +\
+              "/search_contents.pl\?word=" + filename +\
+              "\&searchmode=searchfilesanddirs" +\
+              "\&case=insensitive\&version=unstable" +\
+              "\&arch=i386 2> /dev/null"
+    perform.execute(command)
+    #
+    # Test for multiple page output and handle appropriately
+    #
+    command = "grep page= " + results + " 2>&1 > /dev/null"
     if perform.execute(command) == 0:
-        print "%-50s %-17s" % ("File Path", "Package")
-        print "="*50 + "-" + "="*17
-        sys.stdout.flush()
-        results = tempfile.mktemp()
-        command = "wget --output-document=" + results +\
-		      " http://packages.debian.org/cgi-bin" +\
-		      "/search_contents.pl\?word=" + filename +\
-                      "\&searchmode=searchfilesanddirs" +\
-		      "\&case=insensitive\&version=unstable" +\
-		      "\&arch=i386 2> /dev/null"
-	perform.execute(command)
         #
-        # Test for multiple page output and handle appropriately
+        # Multiple pages of output
         #
-        command = "grep page= " + results + " 2>&1 > /dev/null"
-        if perform.execute(command) == 0:
-            #
-            # Multiple pages of output
-            #
-            command = "cat " + results + " | " +\
-                      "egrep -v '^<|^$|^Packages search page' | " +\
-                      "egrep -v 'page=' | " +\
-                      "perl -p -e 's|<[^>]*>||g;s|<[^>]*$||g;s|^[^<]*>||g;'"
-            perform.execute(command)
-            #
-            # Loop over the other pages
-            #
-            command = "for i in $(grep page= " + results +\
-                      " | perl -p -e 's|.*page=||; s|&.*>||' | sort -u " +\
-                      " | grep -v 1); " +\
-                      "do " +\
-                      "wget --output-document=" + results +\
-                      " http://packages.debian.org/cgi-bin" +\
-                      "/search_contents.pl?page=$i\&word=" + filename +\
-                      "\&version=unstable\&arch=i386" +\
-                      "\&case=insensitive\&searchmode=searchfilesanddirs " +\
-                      " 2> /dev/null; " +\
-                      "cat " + results + " | " +\
-                      "egrep -v '^<|^$|^Packages search page' | " +\
-                      "egrep -v 'page=' | " +\
-                      "perl -p -e 's|<[^>]*>||g;s|<[^>]*$||g;" +\
-                      "s|^[^<]*>||g;';" +\
-                      "done"
-            perform.execute(command)
-        else:
-            #
-            # A single page of output
-            #
-            command = "cat " + results + " | " +\
-                      "egrep -v '^<|^$|^Packages search page' | " +\
-                      "perl -p -e 's|<[^>]*>||g;s|<[^>]*$||g;s|^[^<]*>||g;'"
-            perform.execute(command)
-        if os.path.exists(results): os.remove(results)
+        command = "cat " + results + " | " +\
+                  "egrep -v '^<|^$|^Packages search page|^package |//W3C//' | " +\
+                  "egrep -v 'page=' | " +\
+                  "perl -p -e 's|<[^>]*>||g;s|<[^>]*$||g;s|^[^<]*>||g;'"
+        perform.execute(command)
+        #
+        # Loop over the other pages
+        #
+        command = "for i in $(grep page= " + results +\
+                  " | perl -p -e 's|.*page=||; s|&.*>||' | sort -u " +\
+                  " | grep -v 1); " +\
+                  "do " +\
+                  "wget --timeout=60 --output-document=" + results +\
+                  " http://packages.debian.org/cgi-bin" +\
+                  "/search_contents.pl?page=$i\&word=" + filename +\
+                  "\&version=unstable\&arch=i386" +\
+                  "\&case=insensitive\&searchmode=searchfilesanddirs " +\
+                  " 2> /dev/null; " +\
+                  "cat " + results + " | " +\
+                  "egrep -v '^<|^$|^Packages search page|^package |//W3C//' | " +\
+                  "egrep -v 'page=' | " +\
+                  "perl -p -e 's|<[^>]*>||g;s|<[^>]*$||g;" +\
+                  "s|^[^<]*>||g;';" +\
+                  "done"
+        perform.execute(command)
     else:
-	print "Perhaps the Debian server is temporarily down."
+        #
+        # A single page of output
+        #
+        command = "cat " + results + " | " +\
+                  "egrep -v '^<|^$|^Packages search page|^package |//W3C//' | " +\
+                  "perl -p -e 's|<[^>]*>||g;s|<[^>]*$||g;s|^[^<]*>||g;'"
+        perform.execute(command)
 
+    if os.path.exists(results): os.remove(results)
+
+#------------------------------------------------------------------------
+#
+# FINDPKG
+#
+#------------------------------------------------------------------------
+def do_findpkg(pkg):
+    """Look for a particular pkg at apt-get.org.
+
+    Arguments:
+    pkg		Package to find
+
+    Returns:"""
+    ping_host("www.apt-get.org")
+    #
+    # Print out a suitable heading
+    #
+    print "Lines suitable for /etc/apt/sources.list"
+    print ""
+    sys.stdout.flush()
+    #
+    # Obtain the information from the Apt-Get server
+    #
+    results = tempfile.mktemp()
+    command = "wget --timeout=60 --output-document=" + results +\
+              " http://www.apt-get.org/" +\
+              "search.php\?query=" + pkg +\
+              "\&submit=\&arch%5B%5D=i386\&arch%5B%5D=all " +\
+              "2> /dev/null"
+    perform.execute(command)
+    #
+    # A single page of output
+    #
+    command = "cat " + results + " | " +\
+              "egrep '(^deb|sites and .*packages)' | " +\
+              "perl -p -e 's|<[^>]*>||g;s|<[^>]*$||g;s|^[^<]*>||g;'" +\
+              "| awk '/^deb/{" +\
+              'print "\t", $0;next}/ sites and /' +\
+              '{printf "\\n" ;' +\
+              "print}'"
+    perform.execute(command)
+
+    if os.path.exists(results): os.remove(results)
+
+#------------------------------------------------------------------------
+#
+# RECDOWNLOAD
+#
+#------------------------------------------------------------------------
+
+def do_recdownload(packages):
+    #FIXME: This has problems with virtual packages, FIX THEM!!!
+
+    """Download packages and all dependencies recursively.
+
+        Arguments:
+        packages       List of packages to download recursively
+
+        Returns:
+
+        Author: Juanjo Alvarez <juanjux@yahoo.es>"""
+
+    def get_deps(package):
+        Parse = apt_pkg.ParseTagFile(open("/var/lib/dpkg/available","r"))
+        deplist = []
+        while Parse.Step():
+            if Parse.Section.get("Package")==package:
+                deplist = apt_pkg.ParseDepends(Parse.Section.get("Depends",""))
+                break
+        realdeplist = []
+        if deplist != []:
+            for i in deplist: realdeplist.append( (i[0][0],i[0][1]) )
+        return realdeplist
+
+    def get_deps_recursively(package, packageslist):
+        if not package in packageslist: packageslist.append(package)
+
+        for packageName,versionInfo in get_deps(package):
+            if packageName not in packageslist:
+                packageslist.append(packageName)
+                get_deps_recursively(packageName, packageslist)
+        return packageslist
+
+    packageNames = []
+    dontDownloadList = []
+    for p in packages[:]:
+        # Ignore packages with a "-" at the end so the user can workaround some
+        # dependencies problems (usually in unstable)
+        if p[len(p)-1:] == "-":
+            dontDownloadList.append(p[:-1])
+            packages.remove(p)
+            continue
+
+    print "Calculating all dependencies..."
+    for i in packages:
+        tmp = get_deps_recursively(i,[])
+        for i in tmp:
+            # We don't want dupplicated package names
+            # and we don't want package in the dontDownloadList
+            if i in dontDownloadList: continue
+            if i not in packageNames: packageNames.append(i)
+    print "Packages to download to /var/cache/apt/archives:"
+    for i in packageNames:
+        # We do this because apt-get install dont list the packages to reinstall
+        # if they don't need to be upgraded
+        print i,
+    print "\n"
+
+    command = "apt-get --download-only --reinstall -u install " + perform.concat(packageNames)
+    perform.execute(command, root=1)
+
+def main():
+    do_size([])
+
+if __name__ == '__main__':
+    main()
