@@ -1,7 +1,7 @@
 #
 # WAJIG - Debian Package Management Front End
 #
-# Copyright (c) Graham.Williams@csiro.au
+# Copyright (c) Graham.Williams@togaware.com
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -24,6 +24,7 @@
 #
 #------------------------------------------------------------------------
 import os
+pause = False
 
 setroot = "/bin/su"
 if os.path.exists("/usr/bin/sudo"):
@@ -48,12 +49,17 @@ quiet = ""
 def set_quiet():
     quiet = "> /dev/null"
 
+simulate = 0
 teaching = 0
 
 def set_teaching_level(new_level):
     global teaching
     teaching = new_level
 
+def set_simulate_level(new_level):
+    global simulate
+    simulate = new_level
+
 #------------------------------------------------------------------------
 #
 # concat
@@ -70,23 +76,33 @@ def concat(args):
 # EXECUTE
 #
 #------------------------------------------------------------------------
-def execute(command, root=0, noquiet=0):
-    """Ask the operating system to perform the command
+def execute(command, root=0, noquiet=0, display=1, pipe=0):
+    """Ask the operating system to perform a command.
 
     Arguments:
-    command	A string containing the command and command line options
-    root	If non-zero then root access is required to execute command
-    noquiet	Suppress the use of quiet (in case command has a redirect)
+    
+    COMMAND	A string containing the command and command line options
+    ROOT	If non-zero then root access is required to execute command
+    NOQUIET	Suppress the use of quiet (in case command has a redirect)
+    PIPE	If True then return a file-like object.
 
     Returns:
-    Return status of the command"""
+    
+    Returns either the status of the command or a file-like object
+    if PIPE is True.
 
-    if teaching > 0: print "Performing: " + command
+    Note that the PIPE option was added as a minor modification and has not
+    been fully tested, but is extremely useful in avoiding temporary files."""
+
+    if teaching > 0:
+        print "Performing: " + command
+    elif simulate > 0 and display == 1:
+        print command
     if root > 0:
 	if setroot == "/usr/bin/sudo":
  	    command = setroot + " " + command
         else:
-	    if quiet == "":
+	    if quiet == "" and os.getenv("USER") != "root":
 		print """
 Using `su' but please consider installing `sudo' which is more secure.
 With `sudo' users type their own password and do not need to know the
@@ -101,5 +117,11 @@ to configuring /etc/sudoers.
     else:
       command = command + quiet
     command = "LC_ALL=C; export LC_ALL; set -o noglob; " + command
-    return os.system(command)
-
+    if simulate == 0:
+        if pipe == 0:
+            #if pause:
+            #    command += " | /usr/bin/pager"
+            return os.system(command)
+        else:
+            return os.popen(command)
+    return 0
